import contextlib
import operator

import xbmcplugin

from resources.lib.api.api import API
from resources.lib.const import COMMAND, ROUTE, LANG, SETTINGS, explicit_genres, \
    STRINGS, MEDIA_TYPE, DIR_TYPE, \
    ICON, genre_lang, a_z_threshold_options, RENDER_TYPE
from resources.lib.gui import MoviesItem, SettingsItem, SearchItem, DirectoryItem, TvShowsItem, WatchHistoryItem, \
    MainMenuFolderItem
from resources.lib.gui.renderers import Renderer
from resources.lib.gui.renderers.dialog_renderer import DialogRenderer
from resources.lib.kodilogging import logger
from resources.lib.routing.router import router
from resources.lib.storage.settings import settings
from resources.lib.utils.kodiutils import get_string, get_icon, languages_gui


class DirectoryRenderer(Renderer):
    def __init__(self):
        super(DirectoryRenderer, self).__init__()

    @staticmethod
    @contextlib.contextmanager
    def start_directory(handle, as_type=DIR_TYPE.NONE):
        """Simple context manager that automatically ends the directory."""
        xbmcplugin.setContent(handle, as_type)
        yield
        xbmcplugin.endOfDirectory(handle, cacheToDisc=False)

    @staticmethod
    def main_menu(handle):
        logger.debug('Rendering main menu')
        with DirectoryRenderer.start_directory(handle):
            SearchItem(url=router.get_url(ROUTE.SEARCH))(handle)
            MoviesItem(url=router.get_url(ROUTE.MOVIES))(handle)
            TvShowsItem(url=router.get_url(ROUTE.TV_SHOWS))(handle)
            WatchHistoryItem(url=router.get_url(ROUTE.WATCHED))(handle)
            SettingsItem(url=router.get_url(ROUTE.COMMAND, command=COMMAND.OPEN_SETTINGS))(handle)
        return

    @staticmethod
    def movies(handle):
        with DirectoryRenderer.start_directory(handle):
            DirectoryRenderer.media_menu(handle, MEDIA_TYPE.MOVIE)
            DirectoryItem(title=get_string(LANG.CSFD_TIPS), icon=get_icon(ICON.TV),
                          url=router.get_url(ROUTE.CSFD_TIPS))(handle)

    @staticmethod
    def tv_shows(handle):
        with DirectoryRenderer.start_directory(handle):
            DirectoryRenderer.media_menu(handle, MEDIA_TYPE.TV_SHOW)

    @staticmethod
    def new_releases(media_type):
        return DirectoryItem(title=get_string(LANG.NEWS),
                             icon=get_icon(ICON.NEW),
                             url=router.get_media(media_type, RENDER_TYPE.DEFAULT,
                                                  API.FILTER.new_releases(media_type)))

    @staticmethod
    def new_releases_dubbed(media_type, languages):
        title = get_string(LANG.NEWS_DUBBED)
        title = STRINGS.NEWS_DUBBED_DIR_TITLE.format(title=title, languages='/'.join(languages_gui(languages)))
        return DirectoryItem(title=title,
                             icon=get_icon(ICON.NEW_DUB),
                             url=router.get_media(media_type,
                                                  RENDER_TYPE.DEFAULT,
                                                  API.FILTER.new_releases_dubbed(media_type, languages)))

    @staticmethod
    def popular(media_type):
        return DirectoryItem(title=get_string(LANG.POPULAR),
                             icon=get_icon(ICON.POPULAR),
                             url=router.get_media(media_type,
                                                  RENDER_TYPE.DEFAULT,
                                                  API.FILTER.popular(media_type)))

    @staticmethod
    def last_added(media_type):
        return DirectoryItem(title=get_string(LANG.LAST_ADDED),
                             icon=get_icon(ICON.RECENT),
                             url=router.get_media(media_type,
                                                  RENDER_TYPE.DEFAULT,
                                                  API.FILTER.last_added(media_type)))

    @staticmethod
    def a_z(media_type):
        return DirectoryItem(title=get_string(LANG.A_Z),
                             icon=get_icon(ICON.A_Z),
                             url=router.get_url(ROUTE.A_TO_Z,
                                                media_type=media_type,
                                                letter=STRINGS.EMPTY))

    @staticmethod
    def genres(media_type):
        return DirectoryItem(title=get_string(LANG.GENRE),
                             url=router.get_url(ROUTE.GENRE_MENU,
                                                media_type=media_type),
                             icon=get_icon(ICON.GENRE))

    @staticmethod
    def studios(media_type):
        return DirectoryItem(title=get_string(LANG.BY_STUDIO),
                             icon=get_icon(ICON.STUDIO),
                             url=router.get_url(ROUTE.STUDIO_MENU,
                                                media_type=media_type),
                             )

    @staticmethod
    def media_menu(handle, media_type):
        DirectoryRenderer.popular(media_type)(handle)
        DirectoryRenderer.new_releases(media_type)(handle)
        DirectoryRenderer.new_releases_dubbed(media_type, settings.get_languages(SETTINGS.PREFERRED_LANGUAGE,
                                                                                 SETTINGS.FALLBACK_LANGUAGE))(handle)
        DirectoryRenderer.last_added(media_type)(handle)
        DirectoryRenderer.a_z(media_type)(handle)
        DirectoryRenderer.genres(media_type)(handle)
        DirectoryRenderer.studios(media_type)(handle)

    @staticmethod
    def studio_menu(handle, media_type, studios_count):
        with DirectoryRenderer.start_directory(handle):
            for studio_dict in studios_count:
                studio = studio_dict.get('key')
                DirectoryItem(title=studio,
                              url=router.get_media(media_type,
                                                   RENDER_TYPE.DEFAULT,
                                                   API.FILTER.studio(media_type,
                                                                     studio.encode('utf-8'))))(handle)

    @staticmethod
    def genre_menu(handle, media_type, genres_count):
        genres = []
        for genre_dict in genres_count:
            genre = genre_dict.get('key')
            if not settings.as_bool(SETTINGS.EXPLICIT_CONTENT) and genre in explicit_genres:
                continue
            lang_id = genre_lang.get(genre)
            translated_genre = get_string(lang_id) if lang_id else genre
            genres.append({'name': genre, 'translated': translated_genre.decode('utf-8')})

        genres.sort(key=operator.itemgetter('translated'))
        with DirectoryRenderer.start_directory(handle):
            for genre in genres:
                DirectoryItem(title=genre.get('translated'),
                              url=router.get_media(media_type,
                                                   RENDER_TYPE.DEFAULT,
                                                   API.FILTER.genre(media_type, genre.get('name'))))(handle)

    def a_to_z_menu(self, handle, media_type, letters, letter_counts):
        with DirectoryRenderer.start_directory(handle):
            if letters:
                MainMenuFolderItem(url=router.get_url_command(COMMAND.CLEAR_PATH))(handle)
                search_title = get_string(LANG.SEARCH_FOR_LETTERS).format(letters=letters)
                SearchItem(title=self._a_to_z_title(search_title, letter_counts.get('total')),
                           url=router.get_media(media_type,
                                                RENDER_TYPE.A_Z,
                                                API.FILTER.a_z(media_type, letters)))(handle)
            for letter in letter_counts.get('data'):
                l_key = letter.get('key')
                title = DirectoryRenderer._a_to_z_title(l_key, letter.get('doc_count'))
                if letter.get('doc_count') <= a_z_threshold_options[settings.as_int(SETTINGS.A_Z_THRESHOLD)]:
                    url = router.get_media(media_type, RENDER_TYPE.A_Z, API.FILTER.a_z(media_type, l_key))(
                        handle)
                else:
                    url = router.get_url(ROUTE.A_TO_Z, media_type=media_type, letter=l_key)
                DirectoryItem(title=title, icon=get_icon(ICON.A_Z_2), url=url)(handle)
            if letters:
                MainMenuFolderItem(url=router.get_url_command(COMMAND.CLEAR_PATH))(handle)

    @staticmethod
    def _a_to_z_title(letter, count):
        return STRINGS.A_TO_Z_TITLE.format(letter, str(count))

    # Cannot be more than 1 dir deep due to path history reset
    @staticmethod
    def search(handle, media_type):
        logger.debug('Search dialog opened')
        search_value = DialogRenderer.search()
        xbmcplugin.endOfDirectory(handle, cacheToDisc=False)
        if search_value:
            router.go(router.get_media(media_type,
                                       RENDER_TYPE.SEARCH,
                                       API.FILTER.search(media_type, search_value)))
        else:
            logger.debug('No value for search. Returning.')
            router.replace_route(handle, ROUTE.ROOT)
